#!/usr/bin/env perl
use strict;
use warnings;
use 5.10.0;

BEGIN {
	use Config;
	push @INC, '/opt/linode/longview';
	push @INC, '/opt/linode/longview/lib/perl5';
	push @INC, "/opt/linode/longview/lib/perl5/${Config{archname}}/";
	push @INC, '/opt/linode/longview/usr/include';
	$Net::HTTP::SOCKET_CLASS = 'IO::Socket::INET6';
	require Net::HTTP;
}

use JSON::PP;
use Sys::Hostname;
use LWP::UserAgent;
use Compress::Zlib;
use IO::Socket::INET6;
use Linode::Longview::DataGetter qw(get);
use Linode::Longview::Util qw(constantPush flattenData slurp_file daemonize_self check_already_running $SLEEP_TIME getLogger enableDebugLogging $version);

my $logger = getLogger();

$logger->info("Starting Longview Agent version $version");

$logger->logdie("LongView must be run as root in order to collect data") unless ($< == 0);
my $pid = check_already_running();
$logger->logdie("The LongView agent is already running as PID: $pid") if $pid;

my $confdir = "/etc/linode";
my $apiKeyFile = "$confdir/longview.key";

my $stats = {
	apikey  => scalar(slurp_file($apiKeyFile)),
	version => '1.0',
	payload => [],
};

unless ($stats->{apikey}){
	print "\nNo api key found. Please enter your API Key: ";
	chomp(my $key = <>);
	unless ($key =~ /^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{16}\z$/){
		print "Invalid API Key\n";
		exit 1;
	}
	$stats->{apikey} = $key;
	umask 066;
	mkdir $confdir;
	open my $fh, '>', $apiKeyFile or $logger->logdie("Couldn't open $apiKeyFile for writing: $!");
	print $fh $key;
	close $fh;
}
$logger->logdie("Invalid API key") unless ($stats->{apikey} =~ /^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{16}\z$/);

chown 0,0,$apiKeyFile;
chmod 0600,$apiKeyFile;

my $ua = LWP::UserAgent->new(
	timeout => 10,
	agent   => "Linode Longview 1.0 client: $stats->{apikey}"
);
my $url = 'https://longview1.linode.com/cgi.pl';

my ($debug, $quit) = (0,0);
daemonize_self();
enableDebugLogging() if $debug;

$0 = "Linode-Longview";
$SIG{TERM} = $SIG{INT} = $SIG{HUP} = $SIG{QUIT} = sub { $quit = 1 };

my $data={};
my ($counter, $sleep) = (0, $SLEEP_TIME);
$logger->info("Start up complete");
while (!$quit) {
	$data->{timestamp} = time;
	get($_,$data,) foreach (qw(Processes Ports Memory CPU Network Disk SysInfo Packages));

	constantPush($stats->{payload},$data);
	$data = {};

	$stats->{timestamp} = time;
	my $req = $ua->post(
		$url,
		Content_Type => 'form-data',
		Content => [
			'data' => [
				undef,
				'json.gz',
				'Content-Type' => 'application/json',
				'Content-Encoding' => 'gzip',
				'Content' => Compress::Zlib::memGzip(encode_json($stats))
			]
		]
	);

	$sleep = $SLEEP_TIME;
	if ($req->is_success){
		$logger->debug($req->status_line);
		$logger->debug($req->decoded_content);
		@{$stats->{payload}} = ();
		my $rep = decode_json($req->decoded_content);
		$sleep = $rep->{sleep} if defined $rep->{sleep};
		$logger->logdie("Server has requested this API Key stop sending data") if defined $rep->{die} && $rep->{die} eq "please";
	}
	else{
		$logger->info($req->status_line);
		$logger->info($req->decoded_content);
	}

	$counter++;
	sleep $sleep;
}

